@@ -71,15 +71,16 @@
   (:import-from #:sb-x86-64-asm
                 #:lock #:x66 #:rex #:+rex-b+
                 #:inst-operand-size
+                #:register-p #:reg-num
                 #:machine-ea-p
                 #:machine-ea-base
                 #:machine-ea-index
                 #:machine-ea-disp
-                #:regrm-inst-reg
-                #:regrm-inst-r/m
+                #:regrm-inst-reg #:ext-regrm-inst-reg
+                #:regrm-inst-r/m #:ext-regrm-inst-r/m
                 #:reg-imm-data
                 #:reg/mem-imm-data
-                #:add #:inc #:mov #:lea #:cmp #:xor #:jmp
+                #:add #:xadd #:inc #:mov #:lea #:cmp #:xor #:jmp
                 #:|push| #:|pop| #:|or| #:|call| #:|break|))
 
 (in-package #:sb-aprof)
@@ -183,6 +184,7 @@
          (widetag)
          (size)
          (thread-base-reg 13))
+    (declare (type (or null (unsigned-byte 4)) target-reg))
     (setf (sb-disassem::seg-object seg) component
           (sb-disassem:seg-virtual-location seg) pc
           (sb-disassem:seg-length seg) 64 ; arb
@@ -190,13 +192,12 @@
           (let ((sap (int-sap pc))) (lambda () sap)))
     (macrolet ((fail ()
                 `(progn
-                   (break "fail ~x: ~x '~s' ~s"
-                          pc (logand dchunk #xFF) opcode component)
+                   (cerror "" "fail ~x: ~x '~s' ~s"
+                           pc (logand dchunk #xFF) opcode component)
                    (return-from fail)))
                (advance (newstate)
 ;                 `(format t "~&advance to ~d~%" (setq allocator-state ,newstate))
-                 `(setq allocator-state ,newstate)
-                 )
+                 `(setq allocator-state ,newstate))
                (advance-if (condition newstate)
                  `(if ,condition (advance ,newstate) (fail))))
       (block fail
@@ -223,11 +224,12 @@
              (let* ((opcode (sb-disassem::inst-name inst))
                     (opcode-byte (logand dchunk #xFF))
                     (ea (case opcode
+                          (xadd (ext-regrm-inst-r/m 0 dstate))
                           ;; 64-bit mode 'inc' has reg/mem format,
                           ;; never the single-byte format
                           ((add inc mov lea cmp) (regrm-inst-r/m 0 dstate))))
                     (free-ptr-p
-                     (and (%instancep ea)
+                     (and (machine-ea-p ea)
                           (eql (machine-ea-base ea) thread-base-reg)
                           (eql (machine-ea-disp ea)
                                (ash sb-vm::thread-alloc-region-slot
@@ -235,7 +237,7 @@
                           (not (machine-ea-index ea))))
                     (header-word-p
                      (and target-reg lowtag
-                          (%instancep ea)
+                          (machine-ea-p ea)
                           (eql (machine-ea-base ea) target-reg)
                           (eql (machine-ea-disp ea) (- lowtag))
                           (not (machine-ea-index ea)))))
@@ -251,10 +253,10 @@
                    (#.+state-profiler+
                     (cond
                       ((and (eq opcode 'add) ; possibly stay in +state-profiler+
-                            (%instancep ea)
+                            (machine-ea-p ea)
                             (eql (machine-ea-base ea) 11)
                             (null (machine-ea-index ea))
-                            (eq (machine-ea-disp ea)
+                            (eql (machine-ea-disp ea)
                                 (+ profiler-index sb-vm:n-word-bytes))))
                       (t
                        (advance-if (and (eq opcode 'mov) (pseudoatomic-flag-p))
@@ -267,24 +269,30 @@
                           (t
                            (advance-if (and (eq opcode 'mov) free-ptr-p)
                                        +state-loaded-free-ptr+)
-                           (setq orig-free-ptr-reg (regrm-inst-reg dchunk dstate)))))
+                           (setq orig-free-ptr-reg (reg-num (regrm-inst-reg dchunk dstate))))))
                    (#.+state-loaded-free-ptr+
                     (case opcode
-                      ;; Variable-size alloc can either use LEA or ADD to compute the
+                      ;; Variable-size alloc can use LEA, ADD, or XADD to compute the
                       ;; new free ptr depending on whether the alloc-tn and the size
-                      ;; are in the same register (ADD) or different (LEA).
+                      ;; are in the same register (ADD,XADD) or different (LEA).
                       (lea
-                       (advance-if (and (eql (machine-ea-base ea) orig-free-ptr-reg)
-                                        (or (and (not (machine-ea-index ea))
-                                                 (plusp (machine-ea-disp ea)))
-                                            (and (machine-ea-index ea)
-                                                 (not (machine-ea-disp ea)))))
+                       (advance-if (or (and (not (machine-ea-index ea))
+                                            (plusp (machine-ea-disp ea))
+                                            (eql (machine-ea-base ea) orig-free-ptr-reg))
+                                       (and (not (machine-ea-disp ea))
+                                            (or (eql (machine-ea-index ea) orig-free-ptr-reg)
+                                                (eql (machine-ea-base ea) orig-free-ptr-reg))))
                                    +state-bumped-free-ptr+)
                        (setq free-ptr-reg (regrm-inst-reg dchunk dstate)
                              size (machine-ea-disp ea)))
                       (add
-                       (advance-if (typep ea '(integer 0 15)) +state-bumped-free-ptr+)
+                       (advance-if (register-p ea) +state-bumped-free-ptr+)
                        (setq free-ptr-reg ea))
+                      (xadd
+                       (advance-if (and (register-p ea) (eql (reg-num ea) orig-free-ptr-reg))
+                                   +state-bumped-free-ptr+)
+                       (setq target-reg (reg-num (ext-regrm-inst-reg dchunk dstate))
+                             free-ptr-reg ea))
                       (t (fail))))
                    (#.+state-bumped-free-ptr+
                     (advance-if (and (eq opcode 'cmp)
@@ -309,19 +317,20 @@
                                         (not (machine-ea-index ea))
                                         (<= 0 (machine-ea-disp ea) sb-vm:lowtag-mask))
                                    +state-lowtag-only+)
-                       (setq target-reg (regrm-inst-reg dchunk dstate)
+                       (setq target-reg (reg-num (regrm-inst-reg dchunk dstate))
                              lowtag (machine-ea-disp ea))
                        (when (= lowtag sb-vm:list-pointer-lowtag)
                          (return-from infer-type (values 'list size))))
                       (mov ; widetag stored before ORing in lowtag
-                       (advance-if (and (eq (machine-ea-base ea) orig-free-ptr-reg)
+                       (unless target-reg
+                         (setq target-reg orig-free-ptr-reg))
+                       (advance-if (and (eql (machine-ea-base ea) target-reg)
                                         (not (machine-ea-disp ea))
                                         (not (machine-ea-index ea)))
                                    +state-widetag-only+)
                        (setq widetag (if (eq (inst-operand-size dstate) :qword)
                                          :variable
-                                         (logand (reg/mem-imm-data 0 dstate) #xFF))
-                              target-reg orig-free-ptr-reg))
+                                         (logand (reg/mem-imm-data 0 dstate) #xFF))))
                       (t
                        (fail))))
                    (#.+state-lowtag-only+
@@ -352,7 +361,7 @@
                        ;; then ORs in the function layout, then stores.
                        ;; Also note that FIN must use REG/MEM-IMM-DATA to read the
                        ;; immediate operand where closure uses REG-IMM-DATA.
-                       (when (and (integerp ea)
+                       (when (and (register-p ea)
                                   (eql (logand (reg-imm-data 0 dstate) #xFF)
                                        sb-vm:closure-widetag))
                          (return-from infer-type (values 'closure size)))
@@ -368,18 +377,31 @@
                    (#.+state-low-then-widetag+
                     (infer-layout opcode ea dchunk))
                    (#.+state-widetag-only+
-                    (advance-if (and (eq opcode '|or|)
-                                     ;; TODO: AVER correct register as well
-                                     (not lowtag))
-                                +state-wide-then-lowtag+)
-                    (setq lowtag (reg/mem-imm-data 0 dstate))
-                    (ecase lowtag
-                      (#.sb-vm:other-pointer-lowtag
-                       (return-from infer-type
-                         (values (aref *tag-to-type* widetag) size)))
-                      (#.sb-vm:instance-pointer-lowtag)
-                      (#.sb-vm:fun-pointer-lowtag
-                       (return-from infer-type (values 'function size)))))
+                    (cond ((eq opcode 'mov)
+                           ;; Storing to the word 1 past the header is ignored
+                           ;; and the state is unchanged.
+                           (unless (and (null (machine-ea-index ea))
+                                        (eql (machine-ea-base ea) target-reg)
+                                        (typep (machine-ea-disp ea)
+                                               `(integer ,sb-vm:n-word-bytes
+                                                         (,(* sb-vm:n-word-bytes 2)))))
+                             (fail)))
+                          (t
+                           (advance-if (and (eq opcode '|or|)
+                                            ;; TODO: AVER correct register as well
+                                            (not lowtag))
+                                       +state-wide-then-lowtag+)
+                           (setq lowtag (reg/mem-imm-data 0 dstate))
+                           (ecase lowtag
+                             (#.sb-vm:other-pointer-lowtag
+                              (return-from infer-type
+                                (values (if (eq widetag :variable)
+                                            'unknown
+                                            (aref *tag-to-type* widetag))
+                                        size)))
+                             (#.sb-vm:instance-pointer-lowtag)
+                             (#.sb-vm:fun-pointer-lowtag
+                              (return-from infer-type (values 'function size)))))))
                    (#.+state-wide-then-lowtag+
                     (advance-if (and (eq opcode 'xor) (pseudoatomic-flag-p))
                                 +state-end-pa+))
@@ -401,16 +423,22 @@
                           (t
                            (fail))))
                    (#.+state-result-popped+
-                    (cond ((eq opcode '|or|)
+                    (case opcode
+                      (mov
+                       (advance-if (and (not (machine-ea-index ea))
+                                        (eq (machine-ea-base ea) target-reg)
+                                        (null (machine-ea-disp ea)))
+                                   +state-widetag-only+)
+                       (setq widetag (logand (reg/mem-imm-data 0 dstate) #xFF)))
+                      (|or|
                            (if (eql opcode-byte #x0C)
                                ;; OR AL, $byte (2 byte encoding)
                                (setq lowtag (ldb (byte 8 8) dchunk))
                                ;; OR other-reg, $byte ; (3 byte encoding)
                                (setq lowtag (ldb (byte 8 16) dchunk)))
                            (advance +state-lowtag-only+))
-                          (t
-                           (fail))))
-                   ))))
+                      (t
+                       (fail))))))))
            seg dstate)))))
   (values nil nil))
 
