@@ -18,34 +18,35 @@
   #!+long-float 3.14159265358979323846264338327950288419716939937511l0
   #!-long-float 3.14159265358979323846264338327950288419716939937511d0)
 
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (defun handle-reals (function var)
+    `((((foreach fixnum single-float bignum ratio))
+       (coerce (,function (coerce ,var 'double-float)) 'single-float))
+      ((double-float)
+       (,function ,var))))
+
+  (defun handle-complex (form)
+    `((((foreach (complex double-float) (complex single-float) (complex rational)))
+       ,form))))
+
 ;;; Make these INLINE, since the call to C is at least as compact as a
 ;;; Lisp call, and saves number consing to boot.
-(eval-when (:compile-toplevel :execute)
-
-(sb!xc:defmacro def-math-rtn (name num-args &optional wrapper)
+(defmacro def-math-rtn (name num-args &optional wrapper)
   (let ((function (symbolicate "%" (string-upcase name)))
         (args (loop for i below num-args
                     collect (intern (format nil "ARG~D" i)))))
     `(progn
        (declaim (inline ,function))
        (defun ,function ,args
-         (alien-funcall
-          (extern-alien ,(format nil "~:[~;sb_~]~a" wrapper name)
-                        (function double-float
-                                  ,@(loop repeat num-args
-                                          collect 'double-float)))
-          ,@args)))))
+         (truly-the ;; avoid checking the result
+          ,(type-specifier (fun-type-returns (info :function :type function)))
+          (alien-funcall
+           (extern-alien ,(format nil "~:[~;sb_~]~a" wrapper name)
+                         (function double-float
+                                   ,@(loop repeat num-args
+                                           collect 'double-float)))
+           ,@args))))))
 
-(defun handle-reals (function var)
-  `((((foreach fixnum single-float bignum ratio))
-     (coerce (,function (coerce ,var 'double-float)) 'single-float))
-    ((double-float)
-     (,function ,var))))
-
-(defun handle-complex (form)
-  `((((foreach (complex double-float) (complex single-float) (complex rational)))
-     ,form)))
-) ; EVAL-WHEN
 
 #!+x86 ;; for constant folding
 (macrolet ((def (name ll)
