@@ -120,18 +120,9 @@
     (,definer aref (simple-array (unsigned-byte 8) (*)))
     (,definer sap-ref-8 system-area-pointer)))
 
-;;; FIXME: find out why the comment about SYMBOLICATE below is true
-;;; and fix it, or else replace with SYMBOLICATE.
-;;;
-;;; FIXME: this is cute, but is going to prevent greps for def.*<name>
-;;; from working for (defun ,(make-od-name ...) ...)
 (eval-when (:compile-toplevel :load-toplevel :execute)
   (defun make-od-name (sym1 sym2)
-    ;; "MAKE-NAME" is too generic, but this doesn't do quite what
-    ;; SYMBOLICATE does; MAKE-OD-NAME ("octets definition") it is
-    ;; then.
-    (intern (concatenate 'string (symbol-name sym1) "-" (symbol-name sym2))
-            (symbol-package sym1))))
+    (package-symbolicate (symbol-package sym1) sym1 "-" sym2)))
 
 ;;;; to-octets conversions
 
@@ -404,6 +395,20 @@
                                (simple-array base-char (*))))
                 octets-to-string))
 (defun octets-to-string (vector &key (external-format :default) (start 0) end)
+  "Return a string obtained by decoding VECTOR according to EXTERNAL-FORMAT.
+
+If EXTERNAL-FORMAT is given, it must designate an external format.
+
+If given, START and END must be bounding index designators and
+designate a subsequence of VECTOR that should be decoded.
+
+If some of the octets of VECTOR (or the subsequence bounded by START
+and END) cannot be decoded by EXTERNAL-FORMAT an error of a subtype of
+SB-INT:CHARACTER-DECODING-ERROR is signaled.
+
+Note that for some values of EXTERNAL-FORMAT the length of the
+returned string may be different from the length of VECTOR (or the
+subsequence bounded by START and END)."
   (declare (explicit-check start end :result))
   (with-array-data ((vector vector)
                     (start start)
@@ -421,6 +426,23 @@
                 string-to-octets))
 (defun string-to-octets (string &key (external-format :default)
                          (start 0) end null-terminate)
+  "Return an octet vector that is STRING encoded according to EXTERNAL-FORMAT.
+
+If EXTERNAL-FORMAT is given, it must designate an external format.
+
+If given, START and END must be bounding index designators and
+designate a subsequence of STRING that should be encoded.
+
+If NULL-TERMINATE is true, the returned octet vector ends with an
+additional 0 element that does not correspond to any part of STRING.
+
+If some of the characters of STRING (or the subsequence bounded by
+START and END) cannot be encoded by EXTERNAL-FORMAT an error of a
+subtype of SB-INT:CHARACTER-ENCODING-ERROR is signaled.
+
+Note that for some values of EXTERNAL-FORMAT and NULL-TERMINATE the
+length of the returned vector may be different from the length of
+STRING (or the subsequence bounded by START and END)."
   (declare (explicit-check start end :result))
   (with-array-data ((string string)
                     (start start)
@@ -503,4 +525,4 @@
                          (or (ends-with-p name "->STRING")
                              (ends-with-p name "->STRING*")))
                 (push s macros)))))))
-  sb!impl::*!removable-symbols*)
+  *!removable-symbols*)
