@@ -20,8 +20,6 @@
             sb!vm::zero-tn sb!vm::fp-single-zero-tn sb!vm::fp-double-zero-tn
             sb!vm::zero-offset sb!vm::null-offset sb!vm::code-offset)))
 
-(defconstant +disassem-inst-alignment-bytes+ 4)
-
 
 ;;;; utility functions
 
@@ -629,7 +627,8 @@
   (emit-chooser
    ;; We emit either 12 or 4 bytes, so we maintain 8 byte alignments.
    segment 12 3
-   (lambda (segment posn delta-if-after)
+   (lambda (segment chooser posn delta-if-after)
+     (declare (ignore chooser))
      (let ((delta (funcall calc label posn delta-if-after)))
        (when (<= (- (ash 1 15)) delta (1- (ash 1 15)))
          (emit-back-patch segment 4
