@@ -95,7 +95,8 @@
   ;; something or other that the back end annotates this lvar with
   (info nil)
   (dependent-casts nil)
-  (annotations nil))
+  (annotations nil)
+  (dependent-annotations nil))
 (!set-load-form-method lvar (:xc :target) :ignore-it)
 
 ;;; These are used for annottating a LVAR with information that can't
@@ -111,20 +112,42 @@
   fired)
 
 (defstruct (lvar-modified-annotation
-            (:include lvar-annotation))
+            (:include lvar-annotation)
+            (:copier nil))
   caller)
 
 (defstruct (lvar-hook
-            (:include lvar-annotation))
+            (:include lvar-annotation)
+            (:copier nil))
   (hook #'missing-arg :type function))
 
 (defstruct (lvar-type-spec-annotation
-            (:include lvar-hook)))
+            (:include lvar-hook)
+            (:copier nil)))
 
 (defstruct (lvar-proper-sequence-annotation
-            (:include lvar-annotation))
-  (kind :sequence :type (member proper-list proper-sequence
-                                proper-or-circular-list proper-or-dotted-list)))
+            (:include lvar-annotation)
+            (:copier nil))
+  (kind 'proper-sequence :type (member proper-list proper-sequence
+                                       proper-or-circular-list proper-or-dotted-list)))
+
+(defstruct (lvar-dependent-annotation
+             (:include lvar-annotation)
+             (:copier nil))
+    (deps nil :type list))
+
+(defstruct (lvar-function-designator-annotation
+             (:include lvar-dependent-annotation)
+             (:copier nil))
+  (caller nil :type symbol)
+  (arg-specs nil :type list)
+  (result-specs nil :type list)
+  type)
+
+(defstruct (lvar-function-annotation
+             (:include lvar-annotation)
+             (:copier nil))
+  type)
 
 (defmethod print-object ((x lvar) stream)
   (print-unreadable-object (x stream :type t :identity t)
@@ -318,13 +341,13 @@
                     (:copier nil)
                     (:conc-name block-)
                     (:predicate block-p))
-  ;; a list of all the blocks that are predecessors/successors of this
-  ;; block. In well-formed IR1, most blocks will have one successor.
-  ;; The only exceptions are:
-  ;;  1. component head blocks (any number)
-  ;;  2. blocks ending in an IF (1 or 2)
-  ;;  3. blocks with DELETE-P set (zero)
-  (pred nil :type list)
+    ;; a list of all the blocks that are predecessors/successors of this
+    ;; block. In well-formed IR1, most blocks will have one successor.
+    ;; The only exceptions are:
+    ;;  1. component head blocks (any number)
+    ;;  2. blocks ending in an IF (1 or 2)
+    ;;  3. blocks with DELETE-P set (zero)
+    (pred nil :type list)
   (succ nil :type list)
   ;; the ctran which heads this block (a :BLOCK-START), or NIL when we
   ;; haven't made the start ctran yet (and in the dummy component head
@@ -413,7 +436,7 @@
   ;; During make-host-2, the solution to this is the same hack
   ;; as for everything else: use DEF!STRUCT for IR2-COMPONENT.
   #!+(and (host-feature sb-xc-host) (host-feature sbcl))
-  (declare (sb-ext:muffle-conditions style-warning))
+  (declare (host-sb-ext:muffle-conditions style-warning))
 (def!struct (component (:copier nil)
                        (:constructor
                         make-component
@@ -1552,24 +1575,6 @@
   (array (missing-arg) :type lvar)
   (bound (missing-arg) :type lvar))
 
-(def!struct (dependent-cast (:include cast) (:copier nil))
-  ;; Either LVARs or LEAFs
-  (deps nil :type list))
-
-(def!struct (function-designator-cast (:include dependent-cast)
-                                      (:copier nil))
-  (caller nil :type symbol)
-  (arg-specs nil :type list)
-  (result-specs nil :type list))
-(defprinter (function-designator-cast :identity t)
-  %type-check
-  value
-  asserted-type
-  (deps :test deps)
-  (caller :test caller)
-  (arg-specs :test arg-specs)
-  (result-specs :test result-specs))
-
 ;;; Inserted by ARRAY-CALL-TYPE-DERIVER so that it can be later deleted
 (def!struct (array-index-cast (:include cast) (:copier nil)))
 
