@@ -322,7 +322,7 @@ all_signals_blocked_p(sigset_t *sigset, sigset_t *sigset2,
     if (has_blocked && has_unblocked) {
         char buf[3*64]; // assuming worst case 64 signals present in sigset
         sigset_tostring(sigset, buf, sizeof buf);
-        lose("%s signals partially blocked: {%s}\n", name);
+        lose("%s signals partially blocked: {%s}\n", name, buf);
     }
     if (has_blocked)
         return 1;
@@ -363,7 +363,7 @@ sigaddset_blockable(sigset_t *sigset)
 }
 
 void
-sigaddset_gc(sigset_t *sigset)
+sigaddset_gc(sigset_t __attribute__((unused)) *sigset)
 {
 #ifdef THREADS_USING_GCSIGNAL
     sigaddset(sigset,SIG_STOP_FOR_GC);
@@ -592,7 +592,7 @@ maybe_save_gc_mask_and_block_deferrables(sigset_t *sigset)
  * enabled, without the protection of *GC-INHIBIT* T and there is gc
  * (or stop for gc) pending, but we haven't trapped yet? */
 int
-in_leaving_without_gcing_race_p(struct thread *thread)
+in_leaving_without_gcing_race_p(struct thread __attribute__((unused)) *thread)
 {
     return ((read_TLS(IN_WITHOUT_GCING,thread) != NIL) &&
             (read_TLS(INTERRUPTS_ENABLED,thread) != NIL) &&
@@ -1080,7 +1080,7 @@ interrupt_handle_pending(os_context_t *context)
          * an interrupt arrived during GC (POST-GC, really) it was
          * handled. */
         if (original_pending_handler != data->pending_handler)
-            lose("pending handler changed in gc: %x -> %x.",
+            lose("pending handler changed in gc: %p -> %p.",
                  original_pending_handler, data->pending_handler);
     }
 
@@ -1164,17 +1164,6 @@ interrupt_handle_now(int signal, siginfo_t *info, os_context_t *context)
          * be available; should we copy it or was nobody using it anyway?)
          * then we should convert this to return-elsewhere */
 
-        /* CMUCL comment said "Allocate the SAPs while the interrupts
-         * are still disabled.".  I (dan, 2003.08.21) assume this is
-         * because we're not in pseudoatomic and allocation shouldn't
-         * be interrupted.  In which case it's no longer an issue as
-         * all our allocation from C now goes through a PA wrapper,
-         * but still, doesn't hurt.
-         *
-         * Yeah, but non-gencgc platforms don't really wrap allocation
-         * in PA. MG - 2005-08-29  */
-
-
 #ifndef LISP_FEATURE_SB_SAFEPOINT
         /* Leave deferrable signals blocked, the handler itself will
          * allow signals again when it sees fit. */
@@ -1297,7 +1286,7 @@ store_signal_data_for_later (struct interrupt_data *data, void *handler,
                              siginfo_t *info, os_context_t *context)
 {
     if (data->pending_handler)
-        lose("tried to overwrite pending interrupt handler %x with %x\n",
+        lose("tried to overwrite pending interrupt handler %p with %p",
              data->pending_handler, handler);
     if (!handler)
         lose("tried to defer null interrupt handler\n");
@@ -1415,8 +1404,10 @@ sig_stop_for_gc_handler(int __attribute__((unused)) signal,
     }
 
     if(thread_state(thread)!=STATE_RUNNING) {
-        lose("sig_stop_for_gc_handler: wrong thread state: %ld\n",
-             fixnum_value(thread->state));
+        // macOS warns if OBJ_FMTX is used to format a 'sword_t'
+        // which fixnum_value() returns.
+        lose("sig_stop_for_gc_handler: wrong thread state: %"OBJ_FMTX,
+             (lispobj)fixnum_value(thread->state));
     }
 
     set_thread_state(thread,STATE_STOPPED);
@@ -1432,8 +1423,8 @@ sig_stop_for_gc_handler(int __attribute__((unused)) signal,
     FSHOW_SIGNAL((stderr,"resumed\n"));
 
     if(thread_state(thread)!=STATE_RUNNING) {
-        lose("sig_stop_for_gc_handler: wrong thread state on wakeup: %ld\n",
-             fixnum_value(thread_state(thread)));
+        lose("sig_stop_for_gc_handler: wrong thread state on wakeup: %"OBJ_FMTX,
+             (lispobj)fixnum_value(thread_state(thread)));
     }
 
     if (was_in_lisp) {
@@ -1726,7 +1717,7 @@ handle_guard_page_triggered(os_context_t *context,os_vm_address_t addr)
        addr < CONTROL_STACK_HARD_GUARD_PAGE(th) + os_vm_page_size) {
         fake_foreign_function_call(context);
         lose("Control stack exhausted, fault: %p, PC: %p",
-             addr, *os_context_pc_addr(context));
+             addr, (void*)*os_context_pc_addr(context));
     }
     else if(addr >= CONTROL_STACK_GUARD_PAGE(th) &&
             addr < CONTROL_STACK_GUARD_PAGE(th) + os_vm_page_size) {
@@ -1737,7 +1728,7 @@ handle_guard_page_triggered(os_context_t *context,os_vm_address_t addr)
         if (lose_on_corruption_p || gc_active_p) {
             fake_foreign_function_call(context);
             lose("Control stack exhausted, fault: %p, PC: %p",
-                 addr, *os_context_pc_addr(context));
+                 addr, (void*)*os_context_pc_addr(context));
         }
         if (th->control_stack_guard_page_protected == NIL)
             lose("control_stack_guard_page_protected NIL");
