@@ -170,7 +170,7 @@ wait_for_thread_state_change(struct thread *thread, lispobj state)
             thread->state_not_stopped_waitcount++;
             break;
         default:
-            lose("Invalid state in wait_for_thread_state_change: "OBJ_FMTX"\n", state);
+            lose("Invalid state in wait_for_thread_state_change: %"OBJ_FMTX, state);
         }
     } else {
         wait_sem = NULL;
@@ -974,8 +974,10 @@ void gc_stop_the_world()
                 /* This thread has exited. */
                 gc_assert(thread_state(p)==STATE_DEAD);
             } else if (status) {
-                lose("cannot send suspend thread=%lu: %d, %s\n",
-                     p->os_thread,status,strerror(status));
+                lose("cannot send suspend thread=%lx: %d, %s",
+                     // KLUDGE: assume that os_thread can be cast as long.
+                     // See comment in 'interr.h' about that.
+                     (long)p->os_thread,status,strerror(status));
             }
         }
     }
@@ -1009,8 +1011,8 @@ void gc_start_the_world()
             lispobj state = thread_state(p);
             if (state != STATE_DEAD) {
                 if(state != STATE_STOPPED) {
-                    lose("gc_start_the_world: wrong thread state is %d\n",
-                         fixnum_value(state));
+                    lose("gc_start_the_world: wrong thread state is %"OBJ_FMTX,
+                         (lispobj)fixnum_value(state));
                 }
                 FSHOW_SIGNAL((stderr, "/gc_start_the_world: resuming %lu\n",
                               p->os_thread));
